-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

local holidayFlag =
{
  ["brewfest"             ] = NigglesHolidayBrewfest,
  ["childrensweek"        ] = NigglesHolidayChildrensWeek,
  ["darkmoonfaireterokkar"] = NigglesHolidayDarkmoonFaire,
  ["dayofthedead"         ] = NigglesHolidayDayOfTheDead,
  ["hallowsend"           ] = NigglesHolidayHallowsEnd,
  ["harvestfestival"      ] = NigglesHolidayHarvestFestival,
  ["loveintheair"         ] = NigglesHolidayLoveInTheAir,
  ["lunarfestival"        ] = NigglesHolidayLunarFestival,
  ["midsummer"            ] = NigglesHolidayMidsummer,
  ["noblegarden"          ] = NigglesHolidayNobleGarden,
  ["piratesday"           ] = NigglesHolidayPiratesDay,
  ["winterveil"           ] = NigglesHolidayWinterVeil
};

local holidayList = {};

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--               G  L  O  B  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--               G  L  O  B  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to get a table of the active holidays
--
function NigglesHolidaysGetActive()
  -- Local Variables
  local currentTime;
  local activeHolidays = 0x00;

  -- Search for any active holidays
  currentTime = NigglesMinutes();
  for loop = 1, #holidayList do
    if ((currentTime >= holidayList[loop].start) and
        (currentTime <= holidayList[loop].finish)) then
       activeHolidays = bit.bor(activeHolidays, holidayList[loop].event);
    end
  end

  return activeHolidays;
end

--
-- Function to initialise an array with holiday info for the previous, current
-- and next month
--
function NigglesHolidaysInitialise()
  -- Local Variables
  local _;
  local numEvents;
  local eventCalType;
  local eventDate = {};
  local eventTexture;
  local eventFlag;
  local eventType;
  local numDays;
  local eventSeq;
  local idx;

  -- Clear any existing holiday info
  wipe(holidayList);

  -- Get the current date
  _, eventDate.month, eventDate.day, eventDate.year = CalendarGetDate();

  -- Set the current month to properly initialise the Calendar functions
  CalendarSetAbsMonth(eventDate.month, eventDate.year);

  -- Create a list of holidays in the previous, current and next months
  for monthIdx = -1, 1 do
    -- Work out the current month
    currentMonth = (eventDate.year*12)+eventDate.month+monthIdx-1;
    currentYear  = math.floor(currentMonth/12);
    currentMonth = (currentMonth%12)+1;

    -- Set the current month
    CalendarSetAbsMonth(currentMonth, currentYear);
    _, _, numDays, _ = CalendarGetMonth(0);

    -- Check each day of the month for events
    for dayIdx = 1, numDays do
      numEvents = CalendarGetNumDayEvents(0, dayIdx);
      for eventIdx = 1, numEvents do
        _, eventDate.hour, eventDate.min, eventCalType, eventSeq, eventType,
        eventTexture,  _, _ = CalendarGetDayEvent(0, dayIdx, eventIdx);
        if ((eventCalType == "HOLIDAY") and (eventType == 0)) then
          eventTexture = eventTexture:lower();
          eventFlag    = holidayFlag[eventTexture:gsub("^calendar_", "")];
          if (eventFlag ~= nil) then
            if (eventSeq == "START") then
              -- Add the start event to the holiday info
              table.insert(holidayList,
                {
                  event  = eventFlag;
                  start  = NigglesMinutes(currentYear, currentMonth,
                    dayIdx, eventDate.hour, eventDate.min);
                  finish = 0xFFFFFFFF;
                });
            elseif (eventSeq == "END") then
              -- Search for the most recent start event
              idx = 0;
              for loop = #holidayList, 1, -1 do
                if (eventFlag == holidayList[loop].event) then
                  idx = loop;
                  break;
                end
              end

              -- If a corresponding start event wasn't found
              if (idx == 0) then
                -- ...create one
                table.insert(holidayList,
                  {
                    event = holidayFlag[eventFlag];
                    start = 0;
                  });
                idx = #holidayList;
              end

              -- Add the finish time to the event
              holidayList[idx].finish = NigglesMinutes(currentYear,
                currentMonth, dayIdx, eventDate.hour, eventDate.min);
            end
          end
        end
      end
    end
  end

  return;
end
